
!********************************************************************************
!*   Copyright 1999 -- 2002 C. Kind, M. Reiher, J. Neugebauer, and B. A. Hess,
!*   Chair of Theoretical Chemistry, Friedrich-Alexander-University 
!*   Erlangen-Nuremberg, Egerlandstr. 3, D-91058 Erlangen, Germany.
!*
!*
!*   This file is part of the package SNF.
!*
!*   SNF is free software; you can redistribute it and/or modify
!*   it under the terms of the GNU General Public License as published by
!*   the Free Software Foundation; either version 2 of the License, or
!*   (at your option) any later version.
!*
!*   This program is distributed in the hope that it will be useful,
!*   but WITHOUT ANY WARRANTY; without even the implied warranty of
!*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!*   GNU General Public License for more details.
!*
!*   You should have received a copy of the GNU General Public License
!*   along with this program; if not, write to the Free Software
!*   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
!*
!********************************************************************************
module snf_atcontrib
  use f90_kind

contains

!************************************************************************

subroutine calc_pol_comp(molecule,spectrum,polar_dxyz,pol_contr)
  use snf_molecule, only: type_atom
  use snf_spectrum, only: type_peak
  implicit none
  real(double),intent(in) :: polar_dxyz(:,:)
  type(type_atom), intent(in) :: molecule(:)
  type(type_peak), intent(in) :: spectrum(:)
  real(double),intent(inout):: pol_contr(:,:,:)
  integer :: i,j,k,natoms,nmodes
  nmodes=ubound(spectrum,1)
  if(nmodes.gt.0) then
     natoms=ubound(spectrum(1)%norm_mode,1)/3
     pol_contr=0.d0
     do k=1,6
        do j=1,nmodes
           do i=1,natoms
              pol_contr(i,j,k) = dot_product(polar_dxyz(k,(i*3-2):i*3),spectrum(j)%norm_mode((i*3-2):i*3)) &
                   &        / dsqrt(molecule(i)%mass)
           enddo
        enddo
     enddo
  endif
end subroutine calc_pol_comp

!************************************************************************

subroutine calc_dip_comp(molecule,spectrum,dipol_dxyz,dip_contr)
  use snf_molecule, only: type_atom
  use snf_spectrum, only: type_peak
  implicit none
  real(double),intent(in) :: dipol_dxyz(:,:)
  type(type_atom), intent(in) :: molecule(:)
  type(type_peak), intent(in) :: spectrum(:)
  real(double),intent(inout):: dip_contr(:,:,:)
  integer :: i,j,k,natoms,nmodes
  nmodes=ubound(spectrum,1)
  if(nmodes.gt.0) then
     natoms=ubound(spectrum(1)%norm_mode,1)/3
     dip_contr=0.d0
     do k=1,3
        do j=1,nmodes
           do i=1,natoms
              dip_contr(i,j,k) = dot_product(dipol_dxyz(k,(i*3-2):i*3),spectrum(j)%norm_mode((i*3-2):i*3)) &
                   &        / dsqrt(molecule(i)%mass)
           enddo
        enddo
     enddo
  end if
end subroutine calc_dip_comp

!************************************************************************

subroutine print_dip_comp(molecule,spectrum,dip_contr,out)
  use snf_molecule, only: type_atom
  use snf_spectrum, only: type_peak
  implicit none
  type(type_atom), intent(in) :: molecule(:)
  type(type_peak), intent(in) :: spectrum(:)
  real(double),intent(in) :: dip_contr(:,:,:)
  integer,intent(in) :: out

  write(out,*)'Atomic contributions to IR-Intensity'
  write(out,*)'------------------------------------'
  write(out,*)
  call printtab(molecule,spectrum,dip_contr(:,:,1),out)
end  subroutine print_dip_comp

!************************************************************************

subroutine print_pol_comp(molecule,spectrum,pol_contr,out)
  use snf_molecule, only: type_atom
  use snf_spectrum, only: type_peak
  implicit none
  type(type_atom), intent(in) :: molecule(:)
  type(type_peak), intent(in) :: spectrum(:)
  real(double),intent(in) :: pol_contr(:,:,:)
  integer,intent(in) :: out

  write(out,*)'Atomic contributions to Raman-Intensity'
  write(out,*)'---------------------------------------'
  write(out,*)
  write(out,*)'Component Alpha_xx'
  call printtab(molecule,spectrum,pol_contr(:,:,1),out)
  write(out,*)
  write(out,*)'Component Alpha_xy'
  call printtab(molecule,spectrum,pol_contr(:,:,2),out)
  write(out,*)
  write(out,*)'Component Alpha_xz'
  call printtab(molecule,spectrum,pol_contr(:,:,3),out)
  write(out,*)
  write(out,*)'Component Alpha_yy'
  call printtab(molecule,spectrum,pol_contr(:,:,4),out)
  write(out,*)
  write(out,*)'Component Alpha_yz'
  call printtab(molecule,spectrum,pol_contr(:,:,5),out)
  write(out,*)
  write(out,*)'Component Alpha_zz'
  call printtab(molecule,spectrum,pol_contr(:,:,6),out)
end subroutine print_pol_comp

!************************************************************************

subroutine printtab(molecule,spectrum,mat,out)
!
! modified by J. Neugebauer in order to avoid error messages by ifc
! compiler; never tested the results of this modification
!
  use snf_molecule, only: type_atom
  use snf_spectrum, only: type_peak
  implicit none
  type(type_atom), intent(in) :: molecule(:)
  type(type_peak), intent(in) :: spectrum(:)
  real(double)   , intent(in) :: mat(:,:)
  integer        , intent(in) :: out

  integer :: nrow,ncol,i,j
  ncol=ubound(mat,2)
  nrow=ubound(mat,1)

  write(out,'(300(F10.4))') (spectrum(j)%freq, j=1,ncol)

!  write(out,'(A5,A1,300(F10.5,A1))') 'Freq.','9',((spectrum(j)%freq,'9'),j=1,ncol)
  write(out,'(A5,A1,300(F10.5,A1))') 'Freq.','9',(spectrum(j)%freq,'9',j=1,ncol)
  do i=1,nrow
     write(out,'(A2,I2,A1,300(F10.5,A1))') &
          &    molecule(i)%element,i,'9', (mat(i,j),'9',j=1,ncol)
!          &    molecule(i)%element,i,'9', ((mat(i,j),'9'),j=1,ncol)
  end do
end subroutine printtab

!************************************************************************

end module snf_atcontrib




