
!********************************************************************************
!*   Copyright 1999 -- 2002 C. Kind, M. Reiher, J. Neugebauer, and B. A. Hess,
!*   Chair of Theoretical Chemistry, Friedrich-Alexander-University 
!*   Erlangen-Nuremberg, Egerlandstr. 3, D-91058 Erlangen, Germany.
!*
!*
!*   This file is part of the package SNF.
!*
!*   SNF is free software; you can redistribute it and/or modify
!*   it under the terms of the GNU General Public License as published by
!*   the Free Software Foundation; either version 2 of the License, or
!*   (at your option) any later version.
!*
!*   This program is distributed in the hope that it will be useful,
!*   but WITHOUT ANY WARRANTY; without even the implied warranty of
!*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!*   GNU General Public License for more details.
!*
!*   You should have received a copy of the GNU General Public License
!*   along with this program; if not, write to the Free Software
!*   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
!*
!********************************************************************************
module snf_irred_atoms
  use f90_kind
  use snf_error

  private  
  public :: create_irred_atoms,destroy_irred_atoms,print_irred_atoms
  public :: check_geometry,get_point_grp,is_point_grp,get_rpoint_grp

  !************************************************************************

contains

  function is_point_grp(molecule,threshold,equal,pointgrp) result(valid)
    !----------------------------------------------------------------
    !
    ! written by Carsten Kind, Uni Erlangen, 2001
    !----------------------------------------------------------------
    use snf_molecule,only:type_atom
    use snf_tools,only : getint
    implicit none
    type(type_atom),intent(in) :: molecule(:)
    real(double),intent(in) :: threshold
    character(len=3),intent(in) :: pointgrp
    character(len=132) :: errbuff
    logical :: equal,valid
    integer :: order
    character(len=7) :: symmel
    external equal
    if(pointgrp.eq.'c1 ') then
       valid=.true.
    else if(pointgrp.eq. 'ci ') then
       valid=check_symel(molecule,'sz2_1  ',threshold,equal)
    else if(pointgrp.eq. 'cs ') then
       valid=check_symel(molecule,'sh     ',threshold,equal)
    else if(pointgrp(1:1).eq. 's') then
       order=getint(pointgrp,2)
       write(symmel,'(''sz'',I1,''_1'')')order
       valid=check_symel(molecule,symmel,threshold,equal)
    else if(index('cd',pointgrp(1:1)) .ne.0) then
       order=getint(pointgrp,2)
       write(symmel,'(''cz'',I1,''_1'')')order
       if(check_symel(molecule,symmel,threshold,equal)) then
          if(pointgrp(3:3).eq. 'h') then
             valid=check_symel(molecule,'sh     ',threshold,equal)
          else if(pointgrp(3:3).eq. 'v') then
             valid=check_symel(molecule,'sv2_0  ',threshold,equal)
          else if(pointgrp(3:3).eq. 'd') then
             if(2*order.le.9) then
                write(symmel,'(''sd'',I1,''_0  '')')2*order
             else
                write(symmel,'(''sd'',I2,''_0 '')')2*order
             endif
             valid=check_symel(molecule,symmel,threshold,equal)
          else
             valid=.true.
          endif
          if(pointgrp(1:1).eq. 'd') then
             if(2*order.le.9) then
                write(symmel,'(''cx'',I1,''_0  '')')2*order
             else
                write(symmel,'(''cx'',I2,''_0 '')')2*order
             endif
             valid=valid.and.check_symel(molecule,symmel,threshold,equal)
!              valid = valid .and. check_symel(molecule,'sv2_0  ',threshold,equal)
          endif
       else
          valid=.false.
       endif
    else
       if(pointgrp(1:1).eq.'o') then
          valid=check_symel(molecule,'o1402  ',threshold,equal)
          valid=valid .and. check_symel(molecule,'o1401  ',threshold,equal)
          if(pointgrp.eq.'oh') then
             valid=valid .and. check_symel(molecule,'sh     ',threshold,equal)
          endif
       else if(pointgrp(1:1).eq.'i') then
          valid=check_symel(molecule,'i1502  ',threshold,equal)
          valid=valid .and.check_symel(molecule,'i1501  ',threshold,equal)
          if(pointgrp.eq.'ih') then
             valid=valid .and. check_symel(molecule,'sz2_1  ',threshold,equal)
          endif
       else if(pointgrp(1:1).eq. 't') then
          valid=check_symel(molecule,'o1301  ',threshold,equal)
          valid=valid.and.check_symel(molecule,'o1303  ',threshold,equal)
          if(pointgrp.eq.'td') then
             valid=valid .and.check_symel(molecule,'p1401  ',threshold,equal)
          endif
          if(pointgrp.eq.'th') then
             valid=valid .and.check_symel(molecule,'sh     ',threshold,equal)
          endif
       else
          write(errbuff,*) 'Unknown point-group: ', pointgrp
          call serror('is_point_grp',2,errbuff)
       endif
    end if
  end function is_point_grp

  subroutine get_point_grp(molecule,threshold,equal,pointgrp)
    !----------------------------------------------------------------
    !
    ! written by Carsten Kind, Uni Erlangen, 2001
    !----------------------------------------------------------------
    use snf_molecule,only:type_atom
    implicit none
    type(type_atom),intent(in) :: molecule(:)
    real(double),intent(in) :: threshold
    character(len=3),intent(out) :: pointgrp
    logical :: equal
    logical :: is_sh
    integer :: order
    character(len=7) :: symmel
    external  equal
    order=8
    pointgrp='   '
    is_sh = check_symel(molecule,'sh     ',threshold,equal)
    cnz: do 
       write(symmel,'(''cz'',I1,''_1  '')')order
       if (check_symel(molecule,symmel,threshold,equal)) exit cnz
       order=order-1
    end do cnz
    if(order.eq.5 .and. check_symel(molecule,'i1502  ',threshold,equal)) then
       if(check_symel(molecule,'sz2_1  ',threshold,equal)) then 
          pointgrp='ih'
       else
          pointgrp='i'
       endif
    else if(order.eq.4 .and. check_symel(molecule,'o1402  ',threshold,equal)) then
       if(is_sh) then 
          pointgrp='oh'
       else
          pointgrp='o'
       endif
    else if(order.eq.2 .and. check_symel(molecule,'o1301  ',threshold,equal)) then
       if(is_sh) then 
          pointgrp='th'
       else if(check_symel(molecule,'p1401  ',threshold,equal)) then
          pointgrp='td'
       else
          pointgrp='t'
       endif
    else if(order.ge. 2) then     ! Cnx, Dnx ...
       if(check_symel(molecule,'cx2_0  ',threshold,equal)) then
          ! --> Dn, Dnh, Dnd
          write(pointgrp,'(''d'',I1)')order
          if(2*order.le.9) then
             write(symmel,'(''sd'',I1,''_0  '')')2*order
          else
             write(symmel,'(''sd'',I2,''_0 '')')2*order
          endif
          if(is_sh) then 
             pointgrp(3:3)='h'
          else if(check_symel(molecule,symmel,threshold,equal)) then
             pointgrp(3:3)='d'
          endif
       else                  ! --> Cn, Cnh, Cnv
          write(pointgrp,'(''c'',I1)')order
          if(2*order.le.9) then
             write(symmel,'(''sz'',I1,''_1  '')')2*order
          else
             write(symmel,'(''sz'',I2,''_1 '')')2*order
          endif
!JN          write(*,*)'symmel : ',symmel
          if(is_sh) then
             pointgrp(3:3)='h'
          else if(check_symel(molecule,'sv2_0  ',threshold,equal)) then
             pointgrp(3:3)='v'
          else if(check_symel(molecule,symmel,threshold,equal)) then
             if(2*order.le.9) then
                write(pointgrp,'(''s'',I1,1X)')2*order
             else
                write(pointgrp,'(''s'',I2)')2*order
             endif
          endif
       endif
    else                     ! --> Cs, Ci, C1
       if(is_sh) then
          pointgrp='cs'
       else if(check_symel(molecule,'sz2_1  ',threshold,equal)) then
          pointgrp='ci'
       else 
          pointgrp='c1'
       endif
    endif
  end subroutine get_point_grp

  !************************************************************************

  function check_symel(molecule,symmel,threshold,equal) result(valid)
    !----------------------------------------------------------------
    !
    ! written by Carsten Kind, Uni Erlangen, 2001
    !----------------------------------------------------------------
    use snf_molecule,only:type_atom
    use snf_symmat,only:symel_mat
    implicit none
    logical valid
    type(type_atom),intent(in) :: molecule(:)
    character(LEN=7),intent(in) :: symmel
    real(double),intent(in) :: threshold
    type(type_atom) :: rot_at
    integer :: atom1_ind,atom2_ind,natoms
    logical :: equal
    real(double) :: symmat(3,3)
    external  equal
    natoms = ubound(molecule,1)
    symmat=symel_mat(symmel)
    atom1: do atom1_ind=1,natoms
       !----------------------------------------------------------------
       ! transform atom1 with all symmetry elements of the point group
       !----------------------------------------------------------------
       rot_at%element=molecule(atom1_ind)%element
       rot_at%mass=molecule(atom1_ind)%mass
       rot_at%cartcoord=matmul(symmat,molecule(atom1_ind)%cartcoord)
       !-------------------------------------------------------------
       ! compare coordinates of transformed atom1 and all other atoms
       !-------------------------------------------------------------
       atom2_ind = 1
       atom2: do while (.not.equal(rot_at,molecule(atom2_ind),threshold))
          atom2_ind = atom2_ind+1
          if (atom2_ind > natoms) exit atom2
       enddo atom2
       if(atom2_ind > natoms) then
          valid=.false.
          return
       end if
    end do atom1
    valid = .true.
  end function check_symel

  subroutine check_geometry(molecule,char_table,equal,unit)
    !----------------------------------------------------------------
    !
    ! written by Carsten Kind, Uni Erlangen, 2001
    !----------------------------------------------------------------
    use snf_molecule,only:type_atom
    use snf_char_table,only: type_char_table

    implicit none
    type(type_atom),intent(in) :: molecule(:)
    type(type_char_table),intent(in) :: char_table
    integer,optional :: unit

    integer :: out,i
    logical :: isfine,found!,mcheck
    real(double) :: threshold,max_thres
    real(double),pointer :: thresholds(:)
    Logical :: equal
    external equal

    if(present(unit)) then
       out=unit
    else
       out=6
    endif
    allocate(thresholds(char_table%nsymel))
    isfine=.true.
    found = .false.
    max_thres=0.D0
    sym: do i=2,char_table%nsymel
       threshold = 1.0d-16
       thres: do while(threshold .lt. 1.d-3)
          found = check_symel(molecule,char_table%symel(i),threshold,equal)
          if (found) exit thres
          threshold = threshold * 10.d0
       end do thres
       if(found) then
          thresholds(i)=threshold
          max_thres=max(max_thres,threshold)
       else
          thresholds(i) = -1.d0
       end if
       isfine=isfine .and. found
    end do sym

    if(isfine) then
       write(out,*)
       write(out,'(''   Geometry fits to '',A3,'' symmetry within '',G9.2,'' bohr.'')')Char_table%point_group,max_thres
    else
       write(out,*)' *************************************************'
       write(out,*)' ERROR: Molecule has not desired symmetry: ',Char_table%point_group
       write(out,*)' *************************************************'
       sym1: do i=2,char_table%nsymel
          if(thresholds(i).lt. 0) then
             write(out,'(a,a,a)')'  Symmetry operation ',char_table%symel(i),' failed.'
          else
             write(out,'(3a,e20.10)')' Symmetry operation ',char_table%symel(i),' within:',thresholds(i)
          endif
       end do sym1
    endif

    deallocate(thresholds)
  end subroutine check_geometry

  subroutine create_irred_atoms(molecule,char_table,atom_index,&
     & symm_index,irred_atom_index,nirred_atoms)
    !----------------------------------------------------------------
    ! written by Carsten Kind, Uni Koeln, 1994
    ! modified by Johannes Neugebauer, Uni Erlangen, 2001
    !----------------------------------------------------------------
    use snf_molecule,only:type_atom,atom_equal
    use snf_symmat,only:symel_mat
    use snf_char_table,only: type_char_table

    implicit none
    type(type_atom),intent(in) :: molecule(:)
    type(type_char_table),intent(in) :: char_table
    type(type_atom) :: rot_at
    integer,pointer :: atom_index(:),symm_index(:),irred_atom_index(:)
    integer i,atom1_ind,atom2_ind
    integer natoms,nirred_atoms
    real(double) :: threshold
    logical :: isfine
    !----------------------------------------------------------------
    ! determination of non-redundant atoms ("irreducible atoms")
    !----------------------------------------------------------------
    natoms = ubound(molecule,1)
    allocate(atom_index(natoms),&
         & symm_index(natoms),&
         & irred_atom_index(natoms))
    atom_index(:) = -1
    nirred_atoms = 0
    do atom1_ind=1,natoms
       if(atom_index(atom1_ind) .eq. -1) then
          !-------------------------------------------------------------
          ! neglect dummy atoms
          !-------------------------------------------------------------
          if(molecule(atom1_ind)%element.eq.'Q') cycle
          nirred_atoms=nirred_atoms+1
          irred_atom_index(nirred_atoms)=atom1_ind
          do i=1,char_table%nsymel
             !----------------------------------------------------------------
             ! transform atom1 with all symmetry elements of the point group
             !----------------------------------------------------------------
             isfine = .false.
             threshold = 1.0d-16
             do while(.not. isfine .and. threshold .lt. 1.d-3)
                atom2_ind = 1
                rot_at%element=molecule(atom1_ind)%element
                rot_at%mass=molecule(atom1_ind)%mass
                rot_at%cartcoord=matmul(symel_mat(char_table%symel(i)), &
                     molecule(atom1_ind)%cartcoord)
                !-------------------------------------------------------------
                ! compare coordinates of transformed atom1 and all other atoms
                !-------------------------------------------------------------
                atom2_ind = 1
                atom2: do while (.not. atom_equal(rot_at,molecule(atom2_ind),threshold))
                   atom2_ind = atom2_ind+1
                   if (atom2_ind > natoms) exit atom2
                enddo atom2
                if(atom2_ind > natoms) then
                   isfine = .false.
                   threshold = threshold * 10.d0
                else 
                   isfine=.true.
                endif
             enddo
             if(.not. isfine) then
                write(*,*)' ****************************************************'
                write(*,*)' ERROR: Molecule has not desired symmetry.'
                write(*,*)' Could definitively not map symmetry equivalent atoms'
                write(*,*)' Check failed for symmetry element: ',char_table%symel(i)
                write(*,*)' ****************************************************'
                stop
             endif
             !----------------------------------------------------------------
             ! assign symmetry equivalent atom to irred. atom and 
             ! correspondig symmetry operation if this atom has not been 
             ! assigned to another atom yet
             !----------------------------------------------------------------
             if(atom_index(atom2_ind) .eq. -1) then
                atom_index(atom2_ind) = atom1_ind
                symm_index(atom2_ind) = i
             endif
          enddo
       endif
    enddo
  end subroutine create_irred_atoms

  !************************************************************************

  subroutine print_irred_atoms(atom_index,symm_index,irred_atom_index,nirred_atoms,unit)
    !----------------------------------------------------------------
    !
    ! written by Carsten Kind, Uni Koeln, 1994
    ! modified by Johannes Neugebauer, Uni Erlangen, 05/2001
    !----------------------------------------------------------------
    implicit none
    integer,optional :: unit
    integer,intent(in) :: atom_index(:),symm_index(:),irred_atom_index(:)
    integer :: nirred_atoms,i,out

    if(present(unit)) then
       out=unit
    else
       out=6
    endif
    write(out,*)
    write(out,'('' Information about symmetry redundant atoms'')')
    write(out,'('' ----------------------------------------------'')')
    write(out,'(''   Number of non-redundant atoms:'',i3)') nirred_atoms
    write(out,*)
    write(out,'(''   List of non-redundant atoms:'')')
    write(out,'(3X,16i4)')(irred_atom_index(i),i=1,nirred_atoms)
    write(out,*)
    write(out,'(''   atom No. | related non-redundant atom | symmetry op.'')')
    write(out,'(''   ---------|----------------------------|-------------'')')
    do i=1,ubound(atom_index,1)
       write(out,'(3X,i7,a3,i26,a3,i11)') i,'  |',atom_index(i),'  |',symm_index(i)
    enddo
    write(out,*)   
  end subroutine print_irred_atoms

  !************************************************************************

  subroutine destroy_irred_atoms(atom_index,symm_index,irred_atom_index)
    !----------------------------------------------------------------
    !
    ! written by Carsten Kind, Uni Koeln, 1994
    !----------------------------------------------------------------
    implicit none
    integer,pointer :: atom_index(:),symm_index(:),irred_atom_index(:)

    if (associated(atom_index)) deallocate(atom_index)
    if (associated(symm_index)) deallocate(symm_index)
    if (associated(irred_atom_index)) deallocate(irred_atom_index)
  end subroutine destroy_irred_atoms

  !************************************************************************

  subroutine get_rpoint_grp(molecule,threshold,equal,pointgrp)
    !----------------------------------------------------------------
    ! get the real point group of the molecule, independent of its
    ! current orientation in space 
    ! (subroutine get_point_grp always assumes the Turbomole standard
    ! orientation of the molecule)
    !
    ! written by Johannes Neugebauer, Uni Erlangen, 2001
    !----------------------------------------------------------------
    use snf_molecule,only:type_atom,hauptax_trans,type_molecule,print_molecule
    use snf_char_table,only: pripoint
    implicit none
    type(type_molecule) :: molekel 
    type(type_atom) :: molecule(:)
    type(type_atom),pointer :: mol2(:)
    real(double),intent(in) :: threshold
    real(double) :: mcoord(3),thres,vec(3)
    real(double),pointer :: coord(:,:)
    parameter(thres=1.0d-7)
    integer :: atom,ijk,i,j,k,iat,atom2
    character(len=3),intent(out) :: pointgrp
    character(len=3) :: oripg,newpg
    logical :: equal,haflag
    external  equal

    haflag=.true.
    ijk=0
    allocate(mol2(ubound(molecule,1)))
    mol2=molecule
    call get_point_grp(molecule,threshold,equal,pointgrp)
    write(*,*) 'point group check for original orientation yields: ',pointgrp
    oripg=pointgrp
    newpg=pointgrp

    call hauptax_trans(mol2,molekel,haflag)
    call get_point_grp(mol2,threshold,equal,pointgrp)
!    call print_molecule(mol2)
    write(*,*) 'point group after transformation into I eigenvector basis: ',pointgrp
    if(pripoint(pointgrp).gt.pripoint(oripg)) then
      ijk=123
      newpg=pointgrp
    end if

    mcoord = 0.0d0
    do atom=1,ubound(mol2,1)
       mcoord = mcoord+molecule(atom)%cartcoord*molecule(atom)%mass
    enddo
    mcoord = mcoord/molekel%mass
    !-----------------------------------------------------------------------------
    ! set nuclear coordinates to center of gravity
    !-----------------------------------------------------------------------------
    do atom=1,ubound(mol2,1)
       mol2(atom)%cartcoord = molecule(atom)%cartcoord-mcoord
    enddo   

    mol2=molecule
    call transiax(mol2,molekel,1,3,2)
    call get_point_grp(mol2,threshold,equal,pointgrp)
!    call print_molecule(mol2)
!    write(*,*) 'point group after rotation of molecule: ',pointgrp
    if(pripoint(pointgrp).gt.pripoint(newpg)) then
      write(*,*) 'point group after rotation of molecule: ',pointgrp
      ijk=132
      newpg=pointgrp
    end if

    mol2=molecule
    call transiax(mol2,molekel,2,1,3)
    call get_point_grp(mol2,threshold,equal,pointgrp)
!    call print_molecule(mol2)
!    write(*,*) 'point group after rotation of molecule: ',pointgrp
    if(pripoint(pointgrp).gt.pripoint(newpg)) then
      write(*,*) 'point group after rotation of molecule: ',pointgrp
      ijk=213
      newpg=pointgrp
    end if

    mol2=molecule
    call transiax(mol2,molekel,2,3,1)
    call get_point_grp(mol2,threshold,equal,pointgrp)
!    call print_molecule(mol2)
!    write(*,*) 'point group after rotation of molecule: ',pointgrp
    if(pripoint(pointgrp).gt.pripoint(newpg)) then
      write(*,*) 'point group after rotation of molecule: ',pointgrp
      ijk=231
      newpg=pointgrp
    end if

    mol2=molecule
    call transiax(mol2,molekel,3,1,2)
    call get_point_grp(mol2,threshold,equal,pointgrp)
!    call print_molecule(mol2)
!    write(*,*) 'point group after rotation of molecule: ',pointgrp
    if(pripoint(pointgrp).gt.pripoint(newpg)) then
      write(*,*) 'point group after rotation of molecule: ',pointgrp
      ijk=312
      newpg=pointgrp
    end if

    mol2=molecule
    call transiax(mol2,molekel,3,2,1)
    call get_point_grp(mol2,threshold,equal,pointgrp)
!    call print_molecule(mol2)
!    write(*,*) 'point group after rotation of molecule: ',pointgrp
    if(pripoint(pointgrp).gt.pripoint(newpg)) then
      write(*,*) 'point group after rotation of molecule: ',pointgrp
      ijk=321
      newpg=pointgrp
    end if
    
    mol2=molecule
    if(ijk.gt.0) then
      i=ijk/100
      ijk=ijk-i*100
      j=ijk/10
      ijk=ijk-j*10
      k=ijk
      call transiax(mol2,molekel,i,j,k)
      call get_point_grp(mol2,threshold,equal,pointgrp)
!      write(*,*) 'point group after rotation of molecule: ',pointgrp
      molecule=mol2
      newpg=pointgrp
      oripg=pointgrp
    else
      pointgrp=oripg
      newpg=oripg
    end if

!*****
! Now we definitely (? dieder groups ?) found the correct principal axis of rotation;
! in case of rotation axes of order 3 or larger, we may want to 
! find additional vertical mirror planes; this can be achieved by
! a rotation around the z-axis, such that a particular atom is rotated
! in the x-z-plane
!*****
    
    ijk=0
    mol2=molecule
    allocate(coord(3,ubound(molecule,1)))
    do atom=1,ubound(mol2,1)
      if(abs(molecule(atom)%cartcoord(2)).gt.thres) then
        do iat=1,ubound(mol2,1)
          coord(:,iat)=molecule(iat)%cartcoord(:)
        end do
        call rotzx(coord,ubound(mol2,1),atom)
        do iat=1,ubound(mol2,1)
          mol2(iat)%cartcoord(:)=coord(:,iat)
        end do
        call get_point_grp(mol2,threshold,equal,pointgrp)
        if(pripoint(pointgrp).gt.pripoint(newpg)) then
          write(*,*) 'point group after rotation around z-axis: ',pointgrp
          ijk=atom
          newpg=pointgrp
        end if
      end if
    end do
    if(ijk.gt.0) then
      do atom=1,ubound(mol2,1)
        coord(:,atom)=molecule(atom)%cartcoord(:)
      end do
      call rotzx(coord,ubound(mol2,1),ijk)
      do atom=1,ubound(mol2,1)
        mol2(atom)%cartcoord(:)=coord(:,atom)
      end do
      call get_point_grp(mol2,threshold,equal,pointgrp)
      molecule=mol2
      newpg=pointgrp
      oripg=pointgrp
    else
      pointgrp=oripg
      newpg=pointgrp
    end if

!---------------
! dieder groups 
!---------------

    i=0
    j=0
    mol2=molecule
!    deallocate(coord)
!    allocate(coord(3,ubound(molecule,1)))
    do atom=1,ubound(mol2,1)
      do atom2=atom,ubound(mol2,1)
       if(mol2(atom)%nucl_charge.ne.mol2(atom2)%nucl_charge) cycle
       vec(:)=molecule(atom2)%cartcoord(:)-molecule(atom)%cartcoord(:)
       vec(:)=molecule(atom)%cartcoord(:)+0.5d0*vec(:)
       if(abs(vec(2)).gt.thres.and.abs(vec(3)).lt.1.0d-3) then
        do iat=1,ubound(mol2,1)
          coord(:,iat)=molecule(iat)%cartcoord(:)
        end do
        call rotveczx(coord,ubound(molecule,1),vec)
        do iat=1,ubound(mol2,1)
          mol2(iat)%cartcoord(:)=coord(:,iat)
        end do
        call get_point_grp(mol2,threshold,equal,pointgrp)
        if(pripoint(pointgrp).gt.pripoint(newpg)) then
          write(*,*) 'point group after dieder group check: ',pointgrp
          i=atom
          j=atom2
          newpg=pointgrp
        end if
       end if
      end do
    end do
    if(i.gt.0.and.j.gt.0) then
      vec(:)=molecule(j)%cartcoord(:)-molecule(i)%cartcoord(:)
      vec(:)=molecule(i)%cartcoord(:)+0.5d0*vec(:)
      do iat=1,ubound(mol2,1)
        coord(:,iat)=molecule(iat)%cartcoord(:)
      end do
      call rotveczx(coord,ubound(molecule,1),vec)
      do iat=1,ubound(mol2,1)
        mol2(iat)%cartcoord(:)=coord(:,iat)
      end do
      call get_point_grp(mol2,threshold,equal,pointgrp)
      molecule=mol2
      newpg=pointgrp
      oripg=pointgrp
    else
      pointgrp=oripg
      newpg=pointgrp
    end if

!*****
! d2,d2d ?
!*****

    deallocate(coord)
    deallocate(mol2)
    return

  end subroutine get_rpoint_grp

  !************************************************************************

  subroutine transiax(mol,molekel,ii,jj,kk)
    !----------------------------------------------------------------
    !
    ! written by Johannes Neugebauer, Uni Erlangen, 2001
    !----------------------------------------------------------------
    use snf_molecule,only:type_atom,type_molecule
    implicit none
    type(type_molecule) :: molekel 
    type(type_atom) :: mol(:)
    real(double) :: smat(3,3)
    integer, intent(in) :: ii,jj,kk
    integer :: atom

    smat(:,1)=molekel%p_axes(:,ii)
    smat(:,2)=molekel%p_axes(:,jj)
    smat(:,3)=molekel%p_axes(:,kk)
    do atom=1,ubound(mol,1)
       mol(atom)%cartcoord = matmul(transpose(smat),mol(atom)%cartcoord)
    enddo
    
  end subroutine transiax

  !************************************************************************

    subroutine rotzx(coord,natoms,idum)

    !----------------------------------------------------------------
    !
    ! written by Johannes Neugebauer, Uni Erlangen, 2001
    !----------------------------------------------------------------
    use snf_param
    implicit none
    real(double) :: coord(3,natoms),deg,vec(3),vec2(3),confac
    integer, intent(in) :: idum,natoms
    integer :: i
    
      do i=1,3
        vec(i)=0.0d0
        vec2(i)=0.0d0
      end do
      vec2(1)=coord(1,idum)
      vec2(2)=coord(2,idum)
      vec(1)=1.0d0
      if(vec2(2).lt.0.0d0) vec(1)=-vec(1)
      call getangle(vec2,vec,confac)
      do i=1,3
        vec(i)=0.0d0
      end do
      vec(3)=1.0d0
! wrong direction ?
      confac=-confac*360.0d0/2.0d0/pi
!      write(*,'(a,f11.6,a)') '> rotation around z-axis by ',&
!     & confac,' deg'
      call rotate(coord,natoms,vec,confac)

      end subroutine rotzx

  !************************************************************************

    subroutine rotveczx(coord,natoms,vec0)

    !----------------------------------------------------------------
    !
    ! written by Johannes Neugebauer, Uni Erlangen, 2001
    !----------------------------------------------------------------
    use snf_param
    implicit none
    real(double) :: coord(3,natoms),deg,vec(3),vec2(3),confac,vec0(3)
    integer, intent(in) :: natoms
    integer :: i
    
      do i=1,3
        vec(i)=0.0d0
        vec2(i)=0.0d0
      end do
      vec2(1)=vec0(1)
      vec2(2)=vec0(2)
      vec(1)=1.0d0
      if(vec2(2).lt.0.0d0) vec(1)=-vec(1)
      call getangle(vec2,vec,confac)
      do i=1,3
        vec(i)=0.0d0
      end do
      vec(3)=1.0d0
! wrong direction ?
      confac=-confac*360.0d0/2.0d0/pi
!      write(*,'(a,f11.6,a)') '> rotation around z-axis by ',&
!     & confac,' deg'
      call rotate(coord,natoms,vec,confac)

      end subroutine rotveczx

  !************************************************************************

     subroutine getangle(coord,vec,angle)
 
!-----------------------------------------------------------------
! calculate angle (in [deg]) between vector given by
! coord(3), and plane perpendicular to vec(3)
! (in this version, the subroutine calculates (90- this angle))
!
! written by J. Neugebauer, 2002
!-----------------------------------------------------------------
      implicit none
      real(double) coord(3),angle,vec(3),scr,abs1,abs2,acos
      real(double) sqrt,pi
      parameter(pi=3.1415926535897932384626433832795d0)
      intrinsic acos,sqrt
      scr=scalarprod(coord,vec)
      abs1=sqrt(scalarprod(coord,coord))
      abs2=sqrt(scalarprod(vec,vec))
      angle=(acos(scr/abs1/abs2))
      return
      end subroutine getangle

  !************************************************************************

      subroutine rotate(coord,natoms,vec,angle)
 
!-----------------------------------------------------------------
! rotate molecule around axis given by (0,0,0) and vec(:)
! rotation angle: angle
!
! written by J. Neugebauer, 2002
!-----------------------------------------------------------------
      implicit none
      integer natoms,i,j
      real(double) coord(3,natoms),angle,vec(3),nvec(3),pi
      parameter(pi=3.1415926535897932384626433832795d0)
      angle=angle*2.0d0*pi/360.0d0
      do i=1,natoms
        call rotarb(coord(1,i),vec,angle,nvec)
        do j=1,3
          coord(j,i)=nvec(j)
        end do
      end do
      return
      end subroutine rotate

  !************************************************************************

       subroutine rotarb(p1,ps,phi,p2)
 
   !--------------------------------------------------------------------
   ! performs a rotation Cn^k (==> angle phi = 2*pi*k/n) of a vector
   ! p1(x,y,z) around an axis defined by the points O(0,0,0) and
   ! ps(x,y,z) to yield a new vector p2.
   !
   ! source: Simon L. Altmann, Peter Herzig, Point-Group Theory Tables,
   !         Oxford Science Publications, Clarendon Press, Oxford 1994,
   !         p. 57.
   !
   ! written by J. Neugebauer 05/2001
   !-------------------------------------------------------------------
      use snf_symmat, only: initrtm
 
      implicit none
      real(double) p1(3), p2(3),ps(3),s1,s2,p2p,p2m
      real(double) phi,pi,sin
      real(double) rotm(3,3)
      logical pflag
      parameter (pi = 3.1415926535897932384626433832795d0)
      parameter (pflag=.false.)
      intrinsic sin
      integer i,j
 
       !-------------------------------------------------------------------
       ! calculation of s1 = 2 * [sin(phi/2)]**2
       !-------------------------------------------------------------------
 
      s1=sin(phi/2.0d0)
      s1=2.d0*s1**2
 
       !-------------------------------------------------------------------
       ! calculation of s2 = sin(phi)
       !-------------------------------------------------------------------
 
      s2=sin(phi)
 
       !-------------------------------------------------------------------
       ! set up rotation matrix
       !-------------------------------------------------------------------
 
      call initrtm(ps,phi,rotm)
 
       !-------------------------------------------------------------------
       ! perform rotation p1 ==> p2
       !-------------------------------------------------------------------
 
      do i=1,3
        p2p=0.0d0
        p2m=0.0d0
        do j=1,3
          if(rotm(i,j)*p1(j).le.0) then
            p2m=p2m+rotm(i,j)*p1(j)
          else
            p2p=p2p+rotm(i,j)*p1(j)
          end if
        end do
        p2(i)=p2p+p2m
      end do
 
      return
      end subroutine rotarb

  !************************************************************************
      function scalarprod(vec1,vec2)
      implicit none
      real(double) vec1(3),vec2(3),scalarprod
      integer i
      scalarprod=0.0d0
      do i=1,3
        scalarprod=scalarprod+vec1(i)*vec2(i)
      end do
      end function scalarprod

end module snf_irred_atoms
