
!********************************************************************************
!*   Copyright 1999 -- 2002 C. Kind, M. Reiher, J. Neugebauer, and B. A. Hess,
!*   Chair of Theoretical Chemistry, Friedrich-Alexander-University 
!*   Erlangen-Nuremberg, Egerlandstr. 3, D-91058 Erlangen, Germany.
!*
!*
!*   This file is part of the package SNF.
!*
!*   SNF is free software; you can redistribute it and/or modify
!*   it under the terms of the GNU General Public License as published by
!*   the Free Software Foundation; either version 2 of the License, or
!*   (at your option) any later version.
!*
!*   This program is distributed in the hope that it will be useful,
!*   but WITHOUT ANY WARRANTY; without even the implied warranty of
!*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!*   GNU General Public License for more details.
!*
!*   You should have received a copy of the GNU General Public License
!*   along with this program; if not, write to the Free Software
!*   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
!*
!********************************************************************************
module snf_oldsubs
  !---------------------------------------------------------------------
  ! module which contains old subroutines, which are no longer needed
  ! in snf, but we don't dare to delete them.
  !
  ! written by J. Neugebauer, Uni Erlangen, 2001, 2002
  !---------------------------------------------------------------------
  use f90_kind
  use snf_error, only : serror
  private

  !***********************************************************************

  subroutine old_read_control(ctrl)
    use snf_tools, only: string2integer,string2real
    implicit none
    !----------------------------------------------------------------
    ! read all avaible data from control
    ! this routine is included only for compatibility mode
    !
    ! written by Carsten Kind, Uni Erlangen, 1999
    ! modified by Markus Reiher, Uni Erlangen, Aug. 2001
    ! removed from module snf_control by J. Neugebauer, 28.09.2001
    !----------------------------------------------------------------
    type(type_ctrl),intent(inout) :: ctrl
    character(len=132)          :: buf
    integer :: ucf
    ! --------- read whole control file  -------------------------
    ucf = 42
    buf='  '
    ctrl%symmetry='   '
    open(unit=ucf,file='control',status='unknown')
    do while(index(buf,'$end').eq.0)
       read(ucf,'(a)') buf
       if(index(buf,'$title ').ne.0) then
          read(ucf,'(a)') buf
          ctrl%title=trim(buf)
       else if( index(buf,'$cstep ').ne.0) then
          ctrl%cstep=string2real(buf)
          ctrl%tm_ctrl%cstep=ctrl%cstep
       else if( index(buf,'$symmetry ').ne.0) then
          ctrl%tm_ctrl%symmetry=trim(buf(11:))
       else if( index(buf,'$nf_symmetry ').ne.0) then
          ctrl%symmetry=trim(buf(14:))
       else if( index(buf,'$scfconv ').ne.0) then
          ctrl%scfconv=string2integer(buf)
          ctrl%tm_ctrl%scfconv=ctrl%scfconv
       else if( index(buf,'$scfinstab').ne.0.and.index(buf,'polly').ne.0) then
          ctrl%tm_ctrl%inputpol = 1
          ctrl%raman=.true.
       else if( index(buf,'$scfinstab').ne.0.and.index(buf,'dynpol').ne.0) then
          ctrl%tm_ctrl%inputpol = 2
          ctrl%raman=.true.
          ctrl%dynpol=.true.
       else if( index(buf,'$numderiv ').ne.0) then
          ctrl%numderiv=string2integer(buf)
          ctrl%tm_ctrl%numderiv=ctrl%numderiv
       else if( index(buf,'$ridft ').ne.0) then
          ctrl%mode=2
          ctrl%tm_ctrl%mode=2
       else if( index(buf,'$rimp2 ').ne.0) then
          ctrl%mode=3
          ctrl%tm_ctrl%mode=3
       else if( index(buf,'$coord ').ne.0) then
          ctrl%tm_ctrl%coord=trim(buf(index(buf,'=')+1:))
       endif
    enddo
    close(ucf)
    if(ctrl%raman.and.ctrl%mode.eq.2) then
       call serror('old_read_control',0,'No ridft available for Raman!##&
        &ridft is switched off.')
       ctrl%mode=1
       ctrl%tm_ctrl%mode=1
    endif
    if( ctrl%symmetry .eq. '   ') ctrl%symmetry=ctrl%tm_ctrl%symmetry
  end subroutine old_read_control
 
  !************************************************************************

  subroutine matrix_bfgs_update(hessian,step,gradient,old_gradient)
    !----------------------------------------------------------------
    ! BFGS update (not tested and used!)
    !
    ! written by Carsten Kind, Uni Erlangen, 2000
    !----------------------------------------------------------------
    implicit none
    real(double),intent(inout) :: hessian(:,:)
    real(double),intent(in) ::  step(:),gradient(:),old_gradient(:)
    real(double),pointer :: yvec(:)
    real(double) :: fac,ddot
    integer i,j,nvar
    nvar=ubound(gradient,1)
    allocate(yvec(nvar))
    call dcopy(nvar,gradient,1,yvec,1)
    call daxpy(nvar,-1,old_gradient,1,yvec)
    fac = 1.d0/ddot(nvar,yvec,1,step,1)
    do i=1,nvar
       do j=1,i
          hessian(j,1) = hessian(j,1) + fac*yvec(i)*yvec(j)
       enddo
    enddo
    call dsymv('u',nvar,1.d0,hessian,nvar,step,1,0.d0,yvec,1)
    fac = -1.d0/ddot(nvar,step,1,yvec,1)
    call dsyr('u',nvar,fac,yvec,1,hessian,nvar)
    deallocate(yvec,yvec)
  end subroutine matrix_bfgs_update

  !************************************************************************

  subroutine matrix_psb_update(hessian,step,gradient,old_gradient)
    !----------------------------------------------------------------
    ! PSB update (not tested and used!)
    !
    ! written by Carsten Kind, Uni Erlangen, 2000
    !----------------------------------------------------------------
    real(double),intent(inout) :: hessian(:,:)
    real(double),intent(in) ::  step(:),gradient(:),old_gradient(:)
    real(double),pointer :: tvec(:)
    real(double) :: fac,ddot
    integer nvar
    nvar=ubound(gradient,1)
    allocate(tvec(nvar))
    call dcopy(nvar,gradient,1,tvec,1)
    call daxpy(nvar,-1.d0,old_gradient,1,tvec)
    call dsymv('u',nvar,-1.d0,hessian,1,nvar,step,1,1.d0,tvec,1)
    fac=1.d0/sqrt(ddot(nvar,step,1,step,1))
    call dsyr2('u',nvar,fac,tvec,1,step,1,hessian,nvar)
    fac=-fac*fac * ddot(nvar,tvec,1,step,1)
    call dsyr('u',nvar,fac,step,1,hessian,nvar)
    deallocate(tvec)
  end subroutine matrix_psb_update

  !************************************************************************

  subroutine bfgs_update(hessian,step,gradient,old_gradient)
    !----------------------------------------------------------------
    ! BFGS update (not tested and used!)
    !
    ! written by Carsten Kind, Uni Erlangen, 2000
    !----------------------------------------------------------------
    implicit none
    real(double),intent(inout) :: hessian(:,:)
    real(double),intent(in) ::  step(:),gradient(:),old_gradient(:)
    real(double),pointer :: yvec(:),tvec(:)
    real(double) :: norm_ts,dds
    integer i,j,nvar
    nvar=ubound(gradient,1)
    allocate(yvec(nvar),tvec(nvar))
    tvec(:)=0.d0
    do  j=1,nvar
       do  i=1,nvar
          tvec(i)=tvec(i) + hessian(i,j)*step(j)
       enddo
    enddo
    yvec(:)=gradient(:)-old_gradient(:)
    dds=dot_product(yvec,step)
    !----------------------------------------------------------------
    !  if dds is negative, retention of positive definiteness is not
    !  guaranteed. print a warning and skip update this cycle.
    !----------------------------------------------------------------
    norm_ts=dot_product(tvec,step)
    do  i=2,nvar
       do  j=1,i-1
          hessian(i,j)=hessian(i,j) + &
               & (yvec(i)*yvec(j))/dds - (tvec(i)*tvec(j))/norm_ts
          hessian(j,i)=hessian(i,j)
       enddo
    enddo
    do  i=1,nvar
       hessian(i,i)=hessian(i,i)+(yvec(i)*yvec(i))/dds - (tvec(i)*tvec(i))/norm_ts
    enddo
    deallocate(yvec,tvec)
  end subroutine bfgs_update

  !************************************************************************

  subroutine psb_update(hessian,step,gradient,old_gradient)
    !----------------------------------------------------------------
    ! PSB update (not tested and used!)
    !
    ! written by Carsten Kind, Uni Erlangen, 2000
    !----------------------------------------------------------------
    real(double),intent(inout) :: hessian(:,:)
    real(double),intent(in) ::  step(:),gradient(:),old_gradient(:)
    real(double),pointer :: tvec(:),yvec(:)
    real(double) :: step_norm2,norm_ts,temp
    integer nvar,i,j
    nvar=ubound(gradient,1)
    allocate(tvec(nvar),yvec(nvar))
    tvec(:)=0.d0
    do  j=1,nvar
       do  i=1,nvar
          tvec(i)=tvec(i) + hessian(i,j)*step(j)
       enddo
    enddo

    do i=1,nvar
       yvec(i)=gradient(i)-old_gradient(i)
       tvec(i)=yvec(i)-tvec(i)
    enddo
    step_norm2=dot_product(step,step)
    norm_ts=dot_product(tvec,step)
    norm_ts=norm_ts/step_norm2
    do i=2,nvar
       do j=1,i-1
          temp=tvec(i)*step(j) + step(i)*tvec(j) - step(i)*norm_ts*step(j)
          hessian(i,j)=hessian(i,j)+temp/step_norm2
          hessian(j,i)=hessian(i,j)
       enddo
    enddo
    do  i=1,nvar
       temp=step(i)*(2.0d0*tvec(i) - step(i)*norm_ts)
       hessian(i,i)=hessian(i,i)+temp/step_norm2
    enddo
    deallocate(yvec,tvec)
  end subroutine psb_update

  !************************************************************************

end module snf_oldsubs
