
!********************************************************************************
!*   Copyright 1999 -- 2002 C. Kind, M. Reiher, J. Neugebauer, and B. A. Hess,
!*   Chair of Theoretical Chemistry, Friedrich-Alexander-University 
!*   Erlangen-Nuremberg, Egerlandstr. 3, D-91058 Erlangen, Germany.
!*
!*
!*   This file is part of the package SNF.
!*
!*   SNF is free software; you can redistribute it and/or modify
!*   it under the terms of the GNU General Public License as published by
!*   the Free Software Foundation; either version 2 of the License, or
!*   (at your option) any later version.
!*
!*   This program is distributed in the hope that it will be useful,
!*   but WITHOUT ANY WARRANTY; without even the implied warranty of
!*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!*   GNU General Public License for more details.
!*
!*   You should have received a copy of the GNU General Public License
!*   along with this program; if not, write to the Free Software
!*   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
!*
!********************************************************************************
module snf_rbas
  use f90_kind
  use snf_error
  use snf_tools
  
  private
  public :: type_basis, read_tm_bas, type_basset, type_contr
  
  type type_basis                             ! for one atom type
    type(type_basset),pointer :: basset(:)    ! basis sets for l=0,1,2,3,4, etc.
    character(len=132)    :: basname          ! basis set name
    integer               :: attype           ! atom type (nuclear charge number)
  end type type_basis

  type type_basset
    type(type_contr), pointer :: con(:)       ! contracted basis function
    integer               :: lqn              ! l-quantum number
  end type type_basset

  type type_contr
    real(double),pointer  :: expo(:)          ! exponent for primitive b.f.
    real(double),pointer  :: coef(:)          ! contraction coefficient for
  end type type_contr
                                              ! (prim.,contr.)
  !*********************************************************************
 
contains
 
  !*********************************************************************
 
      subroutine read_tm_bas(basis)

!---------------------------------------------------------------------
! read TURBOMOLE basis file
!
! written by J. Neugebauer, Uni Erlangen, 2002
!---------------------------------------------------------------------
      implicit none 
      type(type_basis),pointer  :: basis(:)
      integer noatyp,i,ir1
      parameter(ir1=1)
      call testfile("basis")
      open(unit=ir1, file="basis")
      call get_no_bas(noatyp,ir1)
      if(noatyp.eq.0) then 
        call serror('read_tm_bas',2,'No basis sets found !')
      else 
        allocate(basis(noatyp))
      end if
      rewind(ir1)
      call getbas(basis,noatyp,ir1)
      close(ir1)
      end subroutine read_tm_bas

!************************************************************************

      subroutine get_no_bas(noatyp,ir1)

!---------------------------------------------------------------------
! determine number of individual TURBOMOLE basis sets
!
! written by J. Neugebauer, Uni Erlangen, 2002
! based on the subroutines from the program "based" by B. A. Hess
!---------------------------------------------------------------------
      use snf_molecule, only : find_nucl_charge

      implicit none 
      integer noatyp,ir1
      character(len=132) buf
      logical bas

      bas=.false.
      write(43,*) 
      write(43,*) 'output from module get_no_bas:'
      write(43,*) 
      noatyp=0
1     READ (ir1,'(A)',END=999) buf
      if (index(buf,'$basis').ne.0) then 
        bas=.true.
        goto 1
      end if
      if(.not.bas) goto 1
      IF (buf(1:1).EQ.'$') GOTO 999
      IF (buf(1:1).EQ.'*') GOTO 2
      GOTO 1
 
2     CONTINUE
      READ (ir1,'(A)',END=999) buf
      IF (buf(1:1).EQ.'*') GOTO 1
      IF (buf(1:1).EQ.'#') GOTO 2
      IF (buf(1:1).EQ.'$') GOTO 999
      write(43,*) 'get_no_bas: found new basis set: '
      write(43,*) buf(1:40)
      noatyp=noatyp+1
      write(43,*) 'no. of basis sets:', noatyp
      goto 2
 
999   CONTINUE
      write(43,*) 
      write(43,*) 'get_no_bas: done'
      write(43,*) 
      end subroutine get_no_bas

!********************************************************************

      subroutine getbas(basis,noatyp,ir1)

!---------------------------------------------------------------------
! read individual TURBOMOLE basis set
!
! written by J. Neugebauer, Uni Erlangen, 2002
! based on the subroutines from the program "based" by B. A. Hess
!---------------------------------------------------------------------
      use snf_molecule, only : find_nucl_charge

      implicit none 
      type(type_basis),pointer :: basis(:)
      integer mxitem,ngty,ndmy,i,ie
      parameter(mxitem=7,ngty=7)
      integer nco(ngty),noatyp,natb,llqn
      integer maxco,maxprim,j,k
      parameter(maxprim=30,maxco=20)
      real(double) expo(maxprim,maxco,ngty),coef(maxprim,maxco,ngty)
      integer npri(maxco,ngty),ir1,mytype
      character(len=2) ty(0:ngty),lty(0:ngty)
      character(len=132) buf
      character(len=40) item(mxitem)
      logical bas
      DATA ty /'SP','S ','P ','D ','F ','G ','H ','I '/
      DATA lty /'sp','s ','p ','d ','f ','g ','h ','i '/

      bas=.false.
      natb=0
1     READ (ir1,'(A)',END=999) buf
      if (index(buf,'$basis').ne.0) then 
        bas=.true.
        goto 1
      end if
      if(.not.bas) goto 1
      IF (buf(1:1).EQ.'$') GOTO 999
      IF (buf(1:1).EQ.'*') GOTO 2
      GOTO 1
 
! new basis found
2     CONTINUE
      READ (ir1,'(A)',END=999) buf
      IF (buf(1:1).EQ.'*') GOTO 1
      IF (buf(1:1).EQ.'#') GOTO 2
      IF (buf(1:1).EQ.'$') GOTO 999
      natb=natb+1
      CALL split(buf,item,mxitem)
      basis(natb)%basname=buf
      write(43,*) basis(natb)%basname
      basis(natb)%attype=find_nucl_charge(item(1)(1:2))
      npri=0
      nco=0
      llqn=0
32    READ (ir1,'(A)',END=999) buf
      IF (buf(1:1).EQ.'*') GOTO 3
      GOTO 32
 
! new basis function
3     READ (ir1,'(A)',END=999) buf
      WRITE (43,*) ' getbas| ->',buf(1:40)
      IF (buf(1:1).EQ.'#') GOTO 3
      if (buf(1:1).EQ.'*') then
        allocate(basis(natb)%basset(llqn))
        write(43,*) 'atom no: ',natb
        write(43,*) 'llqn   : ',llqn
        do i=1,llqn
          basis(natb)%basset(i)%lqn=i
!          write(*,*) ' nco(',i,') = ',nco(i)
          allocate(basis(natb)%basset(i)%con(nco(i)))
          do j=1,nco(i)
!            write(*,*) 'npri(',j,',',i,') = ',npri(j,i)
            allocate(basis(natb)%basset(i)%con(j)%coef(npri(j,i)))
            allocate(basis(natb)%basset(i)%con(j)%expo(npri(j,i)))
            do k=1,npri(j,i)
              basis(natb)%basset(i)%con(j)%expo(k)=expo(k,j,i)
              basis(natb)%basset(i)%con(j)%coef(k)=coef(k,j,i)
              write(43,*) expo(k,j,i),coef(k,j,i)
            end do
          end do
        end do
        goto 2
      end if
      IF (buf(1:1).EQ.'$') GOTO 999
      CALL split(buf,item,mxitem)
      DO mytype=0,ngty
        IF (item(2)(1:2).EQ.lty(mytype)) then
          if(mytype.gt.llqn) llqn=mytype
          if(mytype.lt.llqn) call serror('getbas',2,&
     &'Sort basis functions in ascending order of l-quantum numbers')
          GOTO 4
        end if
      ENDDO
      DO mytype=0,ngty
        if (item(2)(1:2).EQ.ty(mytype)) then
          if(mytype.gt.llqn) llqn=mytype
          if(mytype.lt.llqn) call serror('getbas',2,&
     &'Sort basis functions in ascending order of l-quantum numbers')
          GOTO 4
        end if
      ENDDO
      WRITE (*,*) 'getbas: unknown basis set type ',item(1)(1:2)
      WRITE (*,*) 'input line:'
      WRITE (*,*) buf
      GOTO 999
4     nco(mytype)=nco(mytype)+1
      npri(nco(mytype),mytype)=intgr(item(1),ndmy,20)
      WRITE (43,*) ' getbas| type ',item(2)(1:2),': ',&
     & intgr(item(1),ndmy,20),' primitive(s)'
      DO ie=1,intgr(item(1),ndmy,20)
       READ (ir1,'(A)',END=999) buf
       read(buf,*) expo(ie,nco(mytype),mytype),coef(ie,nco(mytype),mytype)
!       write(43,*) 'nco:',nco(mytype),'ie:',ie
!       write(43,*) expo(ie,nco(mytype),mytype),coef(ie,nco(mytype),mytype)
      ENDDO
      GOTO 3
 
999   CONTINUE
      if(natb.ne.noatyp) call serror('get_no_bas',2,'natb.ne.noatyp')
      end subroutine getbas

!********************************************************************
      
end module snf_rbas 





