 
!********************************************************************************
!*   Copyright 1999 -- 2002 C. Kind, M. Reiher, J. Neugebauer, and B. A. Hess,
!*   Chair of Theoretical Chemistry, Friedrich-Alexander-University
!*   Erlangen-Nuremberg, Egerlandstr. 3, D-91058 Erlangen, Germany.
!*
!*
!*   This file is part of the package SNF.
!*
!*   SNF is free software; you can redistribute it and/or modify
!*   it under the terms of the GNU General Public License as published by
!*   the Free Software Foundation; either version 2 of the License, or
!*   (at your option) any later version.
!*
!*   This program is distributed in the hope that it will be useful,
!*   but WITHOUT ANY WARRANTY; without even the implied warranty of
!*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!*   GNU General Public License for more details.
!*
!*   You should have received a copy of the GNU General Public License
!*   along with this program; if not, write to the Free Software
!*   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
!*
!********************************************************************************
module snf_rdalmol
  use f90_kind
  use snf_error
  use snf_tools
  use snf_param

  private 
  public :: rwdalmol,dal2ctrl,checkdalgeo
 
  !*********************************************************************
    
contains

  !*********************************************************************
 
      subroutine rwdalmol(natoms,ic,co,ir1,mxat,kcharg,rewri)
!
! read/write basic molecular input information from molecule.mol file
!
! written by J. Neugebauer, Uni Erlangen, 2002.
!
      implicit none
      integer mxat,mxsh
      parameter(mxsh=50)
      real(double) :: co(3,mxat)
      integer :: ic(mxat)
      integer :: ir1,lo,lr,lend,iat
      logical :: atomba,basis,intgrl
      character(len=132) :: buf,bbuf
      character(len=2) :: symtxt
      character(len=4) :: rewri
 
      integer kcharg,nat,i,natoms,nsh,nsaos,nnsh,nont,nnsaos
      integer lam(mxat),nco(mxsh,mxat)
      character(len=1) :: charm
      character(len=72) ::  coform

      lo=3
      open(lo,file='molecule.mol')
!*****
! Card 1: INTGRL, BASIS, or ATOMBASIS
!*****
      read (ir1,'(a)',ERR=900,END=900) buf
      lend=2
      IF(buf(1:5).EQ.'BASIS') THEN
       basis=.true.
       lend=3
      ELSE IF (buf(1:6).eq.'ATOMBA') THEN
       atomba=.true.
      ELSE IF (buf(1:6).eq.'INTGRL') THEN
       intgrl=.true.
      ELSE
       call serror('rdalmol',2,'Error! Found wrong keyword: '//buf(1:6))
      END IF
      call llen(buf,lr)
      write(lo,'(a)') buf(1:lr)
      call flush(lo)
!*****
! Card 2,3: (basis,) title
!*****
      do i=1,lend
        read (ir1,'(a)',ERR=900,END=900) buf
        call llen(buf,lr)
        write(lo,'(a)') buf(1:lr)
        call flush(lo)
      end do
!*****
! Card 4:
!*****
      READ (ir1,'(a)',ERR=900,END=900) buf
      call clean(bbuf)
      READ (buf,'(BN,A1,I4,I3,A2,A)') &
     &    charm,nat,kcharg,symtxt,bbuf
      if(symtxt.ne.' 0'.and.symtxt.ne.'0 ') then
        if(symtxt.eq.'  ') then
          call serror('rdalmol',0,'Displaced geometries '// &
     &     'must be calculated in C1 !##Setting symmetry to C1 !')
          symtxt=' 0'
          call llen(bbuf,lr)
          write(buf,8800) charm,nat,kcharg,symtxt,bbuf(1:lr)
8800      FORMAT(A1,I4,I3,A2,A)
        end if
      end if
      call llen(buf,lr)
      write(lo,'(a)') buf(1:lr)
      call flush(lo)                              

!*****
! geometry records
!*****
      natoms=0
      nsaos=0
      nsh=0
      DO iat=1,nat
        CALL rwdalgeo(lo,ir1,1+natoms,lam(iat),nco(1,iat),symtxt,nont, &
     &    mxsh,co,atomba,basis,mxat,ic,rewri)
        natoms=natoms+nont
        if(intgrl) then
          nnsaos=0
          nnsh=0
          CALL rwgtodal(lo,ir1,lam(iat),nco(1,iat),nnsaos,nnsh,mxsh)
          nsh=nsh+nnsh*nont
          nsaos=nsaos+nnsaos*nont
        end if
      END DO
 
 
900   close(ir1)
      close(lo)
      end subroutine rwdalmol


      SUBROUTINE rwdalgeo(lo,ir1,iat,lam,nco,stxt,nont,mxsh,co, &
     & atomba,basis,mxat,ic,rewri)
!
! read/write geometry record for one block
!
! written by J. Neugebauer 10/2001
!
      use snf_molecule, only : find_nucl_charge
      implicit none
      INTEGER ir1,iat,lam,mxsh,nco(mxsh),nat,lo,mxat
      integer ic(mxat)
      real(double) co(3,mxat),zs
! local
      CHARACTER*1 zset
      CHARACTER*4 atn,rewri
      CHARACTER*2 stxt
      CHARACTER*40 atbstring
      CHARACTER*132 buf
      INTEGER irc,nont,i,j,k,nu
      LOGICAL zmattst,atomba,basis
 
      zmattst=.false.
      IF(stxt.ne.' 0'.and.stxt.ne.'  '.and.stxt.ne.'0 ') THEN
        call serror('rwdalgeo',2,'Symmetry adapted input not yet possible')
      END IF
 
! read data for all blocks
 
!*****
! Card 6: ATOMIC DATA:
!         zs(i) - nuclear charge
!         nont  - number of symmetry independent atoms
!         lam   - maximum angular quantum number (s=1,p=2,d=3 etc.)
!         nco   - number of AO-blocks for each l-value
!*****
      if(atomba.or.basis) then
        read (ir1,12,ERR=999,END=999) zset,zs,nont,atbstring
        write (lo,12) zset,zs,nont,atbstring
        call flush(lo)
      else
        read (ir1,11,ERR=999,END=999) zset,zs,nont,lam,(nco(j),j=1,lam)
        write (lo,11) zset,zs,nont,lam,(nco(j),j=1,lam)
        call flush(lo)
      end if
11    FORMAT (A1,F9.0,12I5)
12    FORMAT (A1,F9.0,I5,A)
      IF(zset.ne.' ') THEN
        zmattst=.true.
        call serror('rwdalgeo',2,'Z-matrix input not yet implemented')
      END IF
 
!*****
! Card 7: atomic name and coordinates
!           atn - name of symmetry independent center
!           coo - coordinates of symmetry independent center
!*****
 
! this atom and the symmetry-equivalent atoms
      DO k=iat,iat+nont-1
        READ (ir1,'(A)',ERR=999,END=999) buf
        READ (buf,'(a4)',ERR=999,END=999) atn
        if(rewri.eq.'read') then
          READ (buf(5:),*,ERR=999,END=999) (co(j,k),j=1,3)
          ic(k)=find_nucl_charge(atn(1:2))
        end if
        write (lo,'(a4,3f20.12)') atn,(co(j,k),j=1,3)               
        call flush(lo)
      END DO
      RETURN
999   call serror('rwdalgeo',2,'Error while reading molecular structure input')
      END subroutine rwdalgeo

!************************************************************************
 
      SUBROUTINE rwgtodal(lo,ir1,lam,nco,nsaos,nsh,mxsh)
 
      implicit none
      INTEGER ir1,psy,nuc,nrc,lam,nsaos,lr
      integer mxsh,lo
      CHARACTER*72 coform
      CHARACTER*132 buf
      CHARACTER*1 frmt
      INTEGER i,j,k,ncol,l,m,nco(mxsh),nsh,pb
      LOGICAL free
 
      nsaos=0
      nsh=0
      DO 200 i = 1,lam
       DO 250 j = 1,nco(i)
!*****
! Card 8:
!
! nuc - number of uncontracted shells in AO-block
! nrc - number of contracted shells in AO-block
!*****
        READ (ir1,'(a)',ERR=900,END=900) buf
        call llen(buf,lr)
        write (lo,'(a)') buf(1:lr)
        call flush(lo)
        READ (buf,'(BN,A1,I4,I5)') frmt,nuc,nrc
! i=l - 1
        nsaos=nsaos+nrc*(2*i-1)
        nsh=nsh+nrc
        IF (frmt .EQ. 'F' .OR. frmt .EQ. 'f') THEN
           free = .TRUE.
        ELSE IF (frmt .EQ. 'H' .OR. frmt .EQ. 'h') THEN
           free = .FALSE.
           ncol = 4
        ELSE
           free = .FALSE.
           ncol = 8
        END IF
        IF(nrc.NE.0) THEN
          DO 100 l=1,nuc
!*****
! Card 9:
!*****
            READ(ir1,'(a)',ERR=900,END=900) buf
            call llen(buf,lr)
            write (lo,'(a)') buf(1:lr)
            call flush(lo)
            IF(free) THEN
              call serror('rwgtodal',2,'free input format not supported')
            ELSE
              DO 300 k=2,(nrc-1)/(ncol-1)+1
                READ(ir1,'(a)',ERR=900,END=900) buf
                call llen(buf,lr)
                write (lo,'(a)') buf(1:lr)
                call flush(lo)
300           CONTINUE
            END IF
100       CONTINUE
        ELSE
          call serror('rwgtodal',2,'No coefficients available')
        END IF                                      
250    CONTINUE
200   CONTINUE
 
900   RETURN
      END subroutine rwgtodal

!============================================================================

      subroutine dal2ctrl(ctrl)
!
! create fake Turbomole control file for SNFDC run using information
! from the DALTON input file molecule.mol
! create fake Turbomole coord file as well
!
! based on a subroutine from program "based" by B. A. Hess.
! written by J. Neugebauer, University of Erlangen, 2002
!
      use snf_control, only : type_ctrl
      implicit none
      type(type_ctrl) :: ctrl
      integer li,lo,mxat,natoms,mcharge,nel,ncs,nos,i,mode,nfreq
      parameter(mxat=1000)
      integer,pointer :: ic(:)
      real(double),pointer :: co(:,:)
      character(len=80) form
      character(len=10) :: progtype,calctype
      allocate(ic(mxat),co(3,mxat))
      li=1
      lo=2
      call bfile('molecule.mol')
      open(li, file='molecule.mol.bak')
      call rwdalmol(natoms,ic,co,li,mxat,mcharge,'read')
      call bfile('coord')
      call wcoord('coord',natoms,co,ic)
      close(li)

      nel=mcharge
      do i=1,natoms
        nel=nel-ic(i)
      end do
      nel=abs(nel)
      ncs=nel/2
      nos=mod(nel,2)
      mode=1
      if(ctrl%raman) then
        nfreq=1
        if(associated(ctrl%dynfreq)) nfreq=ubound(ctrl%dynfreq,1)
        write(form,'(a,i2,a)') '(a,',nfreq,'f18.10)'
        if(ctrl%mode.ge.5.and.ctrl%mode.le.8) &
        write(form,'(a,i2,a)') '(a,',nfreq,'(/1f18.10))'
      end if
      call rkeyint('control','$daltonmode',mode)
      call bfile('control')
      call rmfile('control',0)
      open(lo, file='control')
      write(lo,'(A)') '$title'
      write(lo,'(A)') '$operating system linuxcluster'
      write(lo,'(A)') '$symmetry c1'
      write(lo,'(A)') '$scfconv 8'
      write(lo,'(A)') '$rundimensions'
      write(lo,'(A,i4)') '   natoms=',natoms
      call prog_type(mode,progtype,calctype)
      if(index(progtype,'DALTON').ne.0) then
        write(lo,'(A,i3)') '$daltonmode',mode
      end if
      write(lo,'(''$cstep  '',F12.3)')ctrl%cstep
      if(ctrl%raman.and.(.not.ctrl%dynpol)) then
         write(lo,'(a)')'$scfinstab polly'
      else if(ctrl%raman.and.ctrl%dynpol) then
         if(associated(ctrl%dynfreq)) then
           write(lo,form) '$scfinstab dynpol ',(ctrl%dynfreq(i),i=1,&
            &ubound(ctrl%dynfreq,1))
         else
           write(lo,form) '$scfinstab dynpol ',nm2freq/514.5
         end if
      endif
      write(lo,'(A)') '$closed shells'
      write(form,'(A,i1,A)') '(a,i',1+int(log10(dble(ncs))),',10x,a)'
      write(lo,form) ' a       1-',ncs,' ( 2 )'
      if(nos.eq.1) then
        write(lo,'(A)') '$open shells'
        write(lo,'(A,10x,A)') ' a       1',' ( 1 )'
      end if
      write(lo,'(A)') '$end'
      close(lo)
      deallocate(ic,co)
      
      end subroutine dal2ctrl
 
!************************************************************************
 
      subroutine wcoord(tempname,natoms,co,ic)
!
!     write TURBOMOLE coord file
!
!     written by J. Neugebauer, Uni Erlangen, 01/2002
 
      use snf_molecule,only: find_element
      implicit none
      real(double) co(3,natoms)
      integer natoms,ic(natoms),i,lu
      character*(*) tempname
      lu=44
      open (unit=lu,file=tempname,status='unknown',err=1001)
      write(44,'(''$coord'')')
      do i=1,natoms
        write(lu,300) co(1,i), co(2,i), co(3,i), find_element(ic(i))
      enddo
      write(44,'(''$end'')')
      close(lu)
300   format(3f18.10,5x,a2)
1001  return
      end subroutine wcoord
 
!************************************************************************

      subroutine checkdalgeo(molecule,ctrl)
!
! written by J. Neugebauer, Uni Erlangen, 2002
!
      use snf_control, only : type_ctrl
      use snf_molecule, only : type_molecule,type_atom,atom_equal
      use snf_irred_atoms, only : get_rpoint_grp,get_point_grp
      implicit none
      type(type_atom),pointer :: molecule(:)
      type(type_ctrl) :: ctrl
      type(type_molecule) :: molekel
      integer,pointer :: ic(:)
      integer :: natoms,i,j,li,nnat,mcharge
      real(double), pointer :: co(:,:)
      character(len=3) pointgrp,oripg
 
      li=1
      call get_point_grp(molecule,1.D-3,atom_equal,oripg)
      call get_rpoint_grp(molecule,1.D-3,atom_equal,pointgrp)
      write(*,*)
      write(*,*)'Automatic point-group check yields: ',pointgrp
      if(pointgrp.ne.oripg) then
        write(*,*)
        call bfile(ctrl%tm_ctrl%coord)
        call serror('snf_define',0,'Note that a coordinate transformation##was '//&
     & 'necessary to verify this point group.')
        ctrl%symmetry=pointgrp
        natoms=ubound(molecule,1)
        allocate(ic(natoms),co(3,natoms))
        do i=1,natoms
          ic(i)=molecule(i)%nucl_charge
          do j=1,3
            co(j,i)=molecule(i)%cartcoord(j)
          end do
        end do
        call wcoord('coord',natoms,co,ic)
        open(li, file='molecule.mol.bak')
        nnat=natoms
        call rwdalmol(nnat,ic,co,li,natoms,mcharge,'writ')
        close(li)
        deallocate(ic,co)
      else
        ctrl%symmetry=oripg
      end if
      end subroutine checkdalgeo
 
 
!============================================================================                                                                          


end module snf_rdalmol
