!********************************************************************************
!*   Copyright 1999 -- 2002 C. Kind, M. Reiher, J. Neugebauer, and B. A. Hess,
!*   Chair of Theoretical Chemistry, Friedrich-Alexander-University 
!*   Erlangen-Nuremberg, Egerlandstr. 3, D-91058 Erlangen, Germany.
!*
!*
!*   This file is part of the package SNF.
!*
!*   SNF is free software; you can redistribute it and/or modify
!*   it under the terms of the GNU General Public License as published by
!*   the Free Software Foundation; either version 2 of the License, or
!*   (at your option) any later version.
!*
!*   This program is distributed in the hope that it will be useful,
!*   but WITHOUT ANY WARRANTY; without even the implied warranty of
!*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!*   GNU General Public License for more details.
!*
!*   You should have received a copy of the GNU General Public License
!*   along with this program; if not, write to the Free Software
!*   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
!*
!********************************************************************************
module snf_rgrad
  use f90_kind
  use snf_error
  use snf_tools

  private
  public :: read_gradient
  !************************************************************************

contains

  !************************************************************************

      subroutine read_gradient(escf,grad,info)
!----------------------------------------------------------------
! read SCF energy and gradient in Turbomole format
!
! written by J. Neugebauer, Uni Erlangen, 2002 
!----------------------------------------------------------------
      implicit none
      real(double), pointer :: grad(:)
      real(double) :: escf
      integer :: i,j,info,lu,natoms,mxitem,k
      parameter(mxitem=10)
      character(len=128) :: rec,item(mxitem)
      logical :: ex

      info=-2
      natoms=ubound(grad,1)/3
      lu=25
      inquire(file='gradient',exist=ex)
      do i=1,natoms*3
        grad(i)=0.0d0
      end do
      if(.not.ex) then
        info=-1
        return
      end if
      open(lu,file='gradient')
90    read (lu,'(a)',end=99) rec
      call split(rec,item,mxitem)
      if (item(1).eq.'cycle') goto 91
      goto 90
91    continue
      read(item(7),*) escf
      do i=1,natoms
       read (lu,'(A)',end=99) rec
      end do
      k=0
      do i=1,natoms
        read (lu,'(A)',end=99,err=99) rec
        call split(rec,item,mxitem)
        do j=1,3
          k=k+1
          read(item(j),*) grad(k)
        end do
      end do
      info=0
      goto 90
      
99    continue
      close(lu)
      end subroutine read_gradient

end module snf_rgrad
