CC******************************************************************************
C   Copyright 1999 -- 2002 C. Kind, M. Reiher, J. Neugebauer, and B. A. Hess,
C   Chair of Theoretical Chemistry, Friedrich-Alexander-University 
C   Erlangen-Nuremberg, Egerlandstr. 3, D-91058 Erlangen, Germany.
C
C
C   This file is part of the package SNF.
C
C   SNF is free software; you can redistribute it and/or modify
C   it under the terms of the GNU General Public License as published by
C   the Free Software Foundation; either version 2 of the License, or
C   (at your option) any later version.
C
C   This program is distributed in the hope that it will be useful,
C   but WITHOUT ANY WARRANTY; without even the implied warranty of
C   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
C   GNU General Public License for more details.
C
C   You should have received a copy of the GNU General Public License
C   along with this program; if not, write to the Free Software
C   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
C
CC******************************************************************************
C $Id: errex.r,v 1.7 2003/05/16 13:48:06 nbauer Exp $
C error exit routine
C $Log: errex.r,v $
C Revision 1.7  2003/05/16 13:48:06  nbauer
C corrected init_c4
C
C Revision 1.6  2003/02/09 17:27:45  nbauer
C changed snf_restart output
C
C Revision 1.5  2002/02/13 16:23:41  nbauer
C BAH modifications to snfdc
C
C Revision 1.4  2002/02/12 12:50:54  nbauer
C GNU GPL included
C
C Revision 1.3  2002/01/30 19:44:57  nbauer
C MCSCF calculations possible
C
c  Revision 1.1  1994/08/31  14:49:15  hess
c  Initial revision
c
      SUBROUTINE ERREX(TX)
      INCLUDE 'defs.h'
      INCLUDE 'comm.h'
      CHARACTER*(*) TX
      INTEGER info
C#    INTEGER w
C#    DATA w /MPI_COMM_WORLD/
      WRITE (iosl,*) '  errex| program aborted due to error(s).'
      WRITE (iosl,*) '       | ',TX
      call flush(iosl)
      WRITE (*,*) '  errex| program aborted due to error(s).'
      WRITE (*,*) '       | ',TX
C#    CALL MPI_ABORT(w,info)
      CALL termcomm('errex')
** termcomm will not return
      STOP 'miracle in errex'
      END

      subroutine serror(modul,ierr,errmsg,lu)

*----------------------------------------------------------------
*
* error management for anfdefine and anforce
*
* written by J. Neugebauer, Uni Erlangen, 2002
*
* errmsg : original error message
*
* ierr   : ierr = 0 : warning;
*          ierr = 1 : error (without stop)
*          ierr = 2 : fatal error, program stops
*          ierr = 3 : interactive error, must be confirmed by
*                     the user by typing return 
*          otherwise: message
*
* module : name of module (subprogram, function) 
*             in which the warning / error occured
*
* NOTE: If error messages are longer than just one line,
*       '##' may be used to separate the lines (i.e. '##'
*       means <return>).
*----------------------------------------------------------------
      implicit none
      integer maxerrl,maxml,ierr,lu,mxln,nrow
      integer lerr,lmodu,itmp,itmpo,llerr,lrow
      parameter(maxerrl=500,maxml=50,mxln=10)
      character*(*) errmsg,modul
      character*132 errrow(mxln),buf
      character*9 msgtyp
      character*5 lhead,cons
      character*2 newln
      integer i,l1,l2,linel,diff1,diff2,l3,lend,j
      integer diff(mxln)
      parameter(lhead=' *** ')
      parameter(newln='##')
      call llen(errmsg,lerr)
      call llen(modul,lmodu)
      if(lerr.gt.maxerrl) then
        write(*,*)
        write(*,*) '*************************************************'
        write(*,*) '*** WARNING! from module snf_error            ***'
        write(*,*) '*** Error message is truncated since too long ***'
        write(*,*) '*************************************************'
        write(*,*)
      else if(lmodu.gt.maxml) then
        write(*,*)
        write(*,*) '********************************************'
        write(*,*) '*** WARNING! from module snf_error       ***'
        write(*,*) '*** Module name truncated since too long ***'
        write(*,*) '********************************************'
        write(*,*)
      end if
      if ( ierr .eq. 0 ) then
        msgtyp = 'WARNING! '
        cons = 'from '
      elseif (ierr.eq.1.or.ierr.eq.2.or.ierr.eq.3) then
        msgtyp = 'ERROR!!! '
        cons = 'in '
      else
        msgtyp = 'MESSAGE! '
        cons = 'from '
      endif
      i=0
      itmp=0
80    if(index(errmsg(itmp+1:),newln).ne.0) then
        i=i+1
        call clean(errrow(i))
        itmpo=itmp
        itmp=index(errmsg(itmpo+1:),newln)-1+itmp
        if((itmp-itmpo).gt.132) write(*,*) 'Error message too long'
        errrow(i)(1:itmp-itmpo)=errmsg(itmpo+1:itmp)
        itmp=itmp+2
        goto 80
      end if
      nrow=i
      l1=lmodu
      call llen(errmsg(itmp+1:),l2)
      if(nrow.ne.0) then
        call llen(errrow(1),l2)
        do i=2,nrow
          call llen(errrow(i),lrow)
          l2=max(l2,lrow)
        end do
        l2=max(l2,lerr-itmp)
      end if
      call llen(cons,l3)
      linel=max(25+l3+l1,8+l2)
      diff1=linel-25-l3-l1
      diff2=linel-8-(lerr-itmp)
      do i=1,nrow
        call llen(errrow(i),l2)
        diff(i)=linel-8-l2
      end do
      write(lu,*)
      write(lu,'(1x,280a1)') ('*',i=1,linel)
      write(lu,100) lhead,msgtyp,cons(1:l3),' module ',
     * modul(1:lmodu),(' ',i=1,diff1),lhead
      do j=1,nrow
        call llen(errrow(j),l2)
        write(lu,'(150a)') lhead,errrow(j)(1:l2),(' ',i=1,diff(j)),
     *    lhead
      end do
      write(lu,'(150a)') lhead,errmsg(itmp+1:lerr),(' ',i=1,diff2),
     *    lhead
      write(lu,'(1x,180a1)') ('*',i=1,linel)
      write(lu,*)
      if(ierr.eq.2) then
        stop
      else if(ierr.eq.3) then
        write(*,*) 
        write(*,*) '=============================='
        write(*,*) '== Press return to continue =='
        write(*,*) '=============================='
        write(*,*) 
        read(*,'(a)')  buf
      end if
100   format(a5,a9,a,a,150a)
      end
