CC******************************************************************************
C   Copyright 1999 -- 2002 C. Kind, M. Reiher, J. Neugebauer, and B. A. Hess,
C   Chair of Theoretical Chemistry, Friedrich-Alexander-University 
C   Erlangen-Nuremberg, Egerlandstr. 3, D-91058 Erlangen, Germany.
C
C
C   This file is part of the package SNF.
C
C   SNF is free software; you can redistribute it and/or modify
C   it under the terms of the GNU General Public License as published by
C   the Free Software Foundation; either version 2 of the License, or
C   (at your option) any later version.
C
C   This program is distributed in the hope that it will be useful,
C   but WITHOUT ANY WARRANTY; without even the implied warranty of
C   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
C   GNU General Public License for more details.
C
C   You should have received a copy of the GNU General Public License
C   along with this program; if not, write to the Free Software
C   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
C
CC******************************************************************************

      subroutine rbas(mxatt,mxl,mxc,mxp,natt,nlqz,ncon,nprim,coef,expo,
     * tdr,ibc,lqn)

*   
* read TURBOMOLE basis file
* 
* written by J. Neugebauer, Uni Erlangen, 2002
*
*
*  meaning of variables:
*
*  natt                  number of atom types (different nuclear charge numbers)
*  nlqz(natt)            number of different l-quantum numbers in basis set for
*                        this atom
*  ncon(nlqz,natt)       number of contracted basis functions for this l-qn.
*  lqn(nlqz,natt)        l-quantum number for contracted basis function
*  nprim(ncon,nlqz,natt) number of primitives in this contracted basis function
*  expo(::::)            exponent for this primitive
*  coef(::::)            coefficient for this primitive in this contracted b.f.
*  ic(mxatt)             position of basis set for nuclear charge number ic(iat)
*  
      include 'defs.h'   
      character*(255) fname
      character*(*) tdr
      integer mxatt,mxl,mxc,mxp,ltdr,ir1,ibc(mxatt),lqn(mxl,mxatt)
      integer natt,nlqz(mxatt),ncon(mxl,mxatt),nprim(mxc,mxl,mxatt)
      real*8 coef(mxp,mxc,mxl,mxatt),expo(mxp,mxc,mxl,mxatt)
      parameter(ir1=1)
      
      call llen(tdr,ltdr)
      fname=tdr(1:ltdr)//'/basis'
      call testfile(fname)
      write(iosl,*) 'entering rbas'
      call flush(iosl)
      open(unit=ir1, file=fname)
      call getnobas(natt,ir1)
      if(natt.eq.0) call errex('no basis sets found')
      rewind(ir1)
      call getbas(mxatt,mxl,mxc,mxp,natt,nlqz,ncon,nprim,coef,expo,
     * ir1,ibc,lqn)
      close(ir1)
      end

C******************************************************************************

      subroutine testfile(rfile)
*
*     if rfile does not exist, then stop.
*
*     written by J. Neugebauer, University of Erlangen, 2001
 
      include 'defs.h'
      character*(*) rfile
      integer lrf
      logical ex
 
      call llen(rfile,lrf)
      inquire(file=rfile(1:lrf),exist=ex)
      if(.not.ex) then
        write(iosl,'(3a)') '** testfile: file ',rfile(1:lrf),
     &   ' does not exist.'
        call flush(iosl)
        stop
      end if
      end

C******************************************************************************

      subroutine getnobas(noatyp,ir1)

C---------------------------------------------------------------------
C determine number of individual TURBOMOLE basis sets
C
C written by J. Neugebauer, Uni Erlangen, 2002
C based on the subroutines from the program "based" by B. A. Hess
C---------------------------------------------------------------------

      include 'defs.h'
      integer noatyp,ir1
      character*132 buf
      logical bas
 
      bas=.false.
      write(iosl,*)
      write(iosl,*) 'output from module get_no_bas:'
      write(iosl,*)
      noatyp=0
1     READ (ir1,'(A)',END=999) buf
      if (index(buf,'$basis').ne.0) then
        bas=.true.
        goto 1
      end if
      if(.not.bas) goto 1
      IF (buf(1:1).EQ.'$') GOTO 999
      IF (buf(1:1).EQ.'*') GOTO 2
      GOTO 1
 
2     CONTINUE
      READ (ir1,'(A)',END=999) buf
      IF (buf(1:1).EQ.'*') GOTO 1
      IF (buf(1:1).EQ.'#') GOTO 2
      IF (buf(1:1).EQ.'$') GOTO 999
      write(iosl,*) 'get_no_bas: found new basis set: '
      write(iosl,*) buf(1:40)
      noatyp=noatyp+1
      write(iosl,*) 'no. of basis sets:', noatyp
      goto 2
 
999   CONTINUE
      write(iosl,*)
      write(iosl,*) 'get_no_bas: done'
      write(iosl,*)
      call flush(iosl)
      end 

C******************************************************************************

      subroutine getbas(mxatt,mxl,mxc,mxp,natt,nlqz,ncon,nprim,
     * coef,expo,ir1,ibc,lqn)

C---------------------------------------------------------------------
C read individual TURBOMOLE basis set
C
C written by J. Neugebauer, Uni Erlangen, 2002
C based on the subroutines from the program "based" by B. A. Hess
C---------------------------------------------------------------------
      include 'defs.h'
      integer mxatt,mxl,mxc,mxp,natt
      integer nlqz(mxatt),ncon(mxl,mxatt),nprim(mxc,mxl,mxatt)
      real*8 coef(mxp,mxc,mxl,mxatt),expo(mxp,mxc,mxl,mxatt)
      integer mxitem,ndmy,i,ie,ibc(mxatt),find_nucl_charge
      integer nco(mxl),natb,llqn,lqn(mxl,mxatt),ngty,intgr
      parameter(mxitem=7,ngty=7)
      integer j,k,l
      integer ir1,mytype
      character*2 ty(0:ngty),lty(0:ngty)
      character*132 buf
      character*40 item(mxitem)
      logical bas
      external find_nucl_charge,intgr
      DATA ty /'SP','S ','P ','D ','F ','G ','H ','I '/
      DATA lty /'sp','s ','p ','d ','f ','g ','h ','i '/
 
*     initialize arrays
      do i=1,mxatt
        ibc(i)=0
        nlqz(i)=0
        do j=1,mxl
          ncon(j,i)=0
          lqn(j,i)=0
          do k=1,mxc
            nprim(k,j,i)=0
            do l=1,mxp
              coef(l,k,j,i)=0.0d0
              expo(l,k,j,i)=0.0d0
            end do
          end do
        end do
      end do
      bas=.false.
      natb=0
1     READ (ir1,'(A)',END=999) buf
      if (index(buf,'$basis').ne.0) then
        bas=.true.
        goto 1
      end if
      if(.not.bas) goto 1
      IF (buf(1:1).EQ.'$') GOTO 999
      IF (buf(1:1).EQ.'*') GOTO 2
      GOTO 1
 
* new basis found
2     CONTINUE
      READ (ir1,'(A)',END=999) buf
      IF (buf(1:1).EQ.'*') GOTO 1
      IF (buf(1:1).EQ.'#') GOTO 2
      IF (buf(1:1).EQ.'$') GOTO 999
      natb=natb+1
      CALL split(buf,item,mxitem)
      write(iosl,*) buf(1:40)
      call flush(iosl)
      ibc(natb)=find_nucl_charge(item(1)(1:2))
      llqn=0
32    READ (ir1,'(A)',END=999) buf
      IF (buf(1:1).EQ.'*') GOTO 3
      GOTO 32
 
* new basis function
3     READ (ir1,'(A)',END=999) buf
      WRITE (iosl,*) ' getbas| ->',buf(1:40)
      call flush(iosl)
      IF (buf(1:1).EQ.'#') GOTO 3
      if (buf(1:1).EQ.'*') then
        nlqz(natb)=llqn
        write(iosl,*) 'atom no: ',natb
        write(iosl,*) 'llqn   : ',llqn
        call flush(iosl)
        do i=1,llqn
          lqn(i,natb)=i
        end do
        goto 2
      end if
      IF (buf(1:1).EQ.'$') GOTO 999
      CALL split(buf,item,mxitem)
      DO mytype=0,mxl
        IF (item(2)(1:2).EQ.lty(mytype)) then
          if(mytype.gt.llqn) llqn=mytype
          if(mytype.lt.llqn) call errex(
     *'Sort basis functions in ascending order of l-quantum numbers')
          GOTO 4
        end if
      ENDDO
      DO mytype=0,mxl
        if (item(2)(1:2).EQ.ty(mytype)) then
          if(mytype.gt.llqn) llqn=mytype
          if(mytype.lt.llqn) call errex(
     *'Sort basis functions in ascending order of l-quantum numbers')
          GOTO 4
        end if
      ENDDO
      WRITE (iosl,*) 'getbas: unknown basis set type ',item(1)(1:2)
      WRITE (iosl,*) 'input line:'
      WRITE (iosl,*) buf
      call flush(iosl)
      GOTO 999
4     ncon(mytype,natb)=ncon(mytype,natb)+1
      nprim(ncon(mytype,natb),mytype,natb)=intgr(item(1),ndmy,20)
      WRITE (iosl,*) ' getbas| type ',item(2)(1:2),': ',
     * intgr(item(1),ndmy,20),' primitive(s)'
      call flush(iosl)
      DO ie=1,intgr(item(1),ndmy,20)
       READ (ir1,'(A)',END=999) buf
       read(buf,*) expo(ie,ncon(mytype,natb),mytype,natb),
     *  coef(ie,ncon(mytype,natb),mytype,natb)
       write(iosl,*) expo(ie,ncon(mytype,natb),mytype,natb),
     *  coef(ie,ncon(mytype,natb),mytype,natb)
      ENDDO
      call flush(iosl)
      GOTO 3
 
999   CONTINUE
      if(natb.ne.natt) call errex('natb.ne.natt in getbas')
      end
